Prompt = require("prompt")
Request = require("request")
Utils = require("../utils")
recursive = require("recursive-readdir")
fs = require("fs")
Document = require("@hunterbrowning/ruleone-models").Document
Async = require("async")
Path = require("path")
colors = require("colors")

# Demo data
theCompanyData = require("../../dev-data/company")
adminData = require("../../dev-data/admins")
userData = require("../../dev-data/users")
caseData = require("../../dev-data/cases")
activityData = require("../../dev-data/activity")
custodianData = require("../../dev-data/custodians")
tagsData = require("../../dev-data/tags")

API_URL = process.env.RO_API_URL
UPLOAD_API_URL = process.env.RO_UPLOAD_API_URL

module.exports = ->
   # High Level Objects
   @sessionToken = null
   @auth = null
   insertedCompany = theCompanyData()
   insertedCases = {}
   insertedUsers = {}
   insertedCustodians = {}
   uploadSetCount = 0
   tags = tagsData().map (tag) ->
      return tag['tag']

   console.log "*** SETTING UP COMPANIES ***".green
   for admin in adminData()
      if admin.company_name == insertedCompany.name
         insertedCompany['admin'] = admin
         break
   console.log "sending data: ", insertedCompany
   Request {
      url: "#{API_URL}/setup-company"
      method: "POST"
      body: insertedCompany
      json: true
   }, (err, res, body) =>
      throw err if err
      Utils.printResponse(res, body)
      insertedCompany['id'] = body.user.companyId
      @sessionToken = body.sessionToken
      @auth = body
      uploadUsers()
   
   uploadUsers = =>
      console.log "\n"
      console.log "*** UPLOADING USERS ***".green
      userProcessed = 0
      for person in userData()
         Request {
            url: "#{API_URL}/users"
            method: "POST"
            body: {
               first_name: person.first_name
               last_name: person.last_name
               email: person.email
               password: person.password
               company_id: insertedCompany.id
            }
            json: true
         }, (err, res, body) ->
            console.log "err: ", err if err
            process.exit() if err
            insertedUsers[body.user.email] = body.user.id
            userProcessed++
            if userProcessed == userData().length
               console.log "Users: ", insertedUsers
               uploadCases()

   uploadCases = =>
      console.log "\n"
      console.log "*** UPLOAD CASES ***".green
      casesProcessed = 0
      for suit in caseData()
         Request {
            url: "#{API_URL}/companies/#{insertedCompany.id}/cases?sessionToken=#{@sessionToken}"
            method: "POST"
            body: {
               name: suit.name
               case_number: suit.case_number
               tags: tags
            }
            json: true
         }, (err, res, body) ->
            console.log "err: ", err if err
            throw err if err
            insertedCases[body.case_number] = body.id
            casesProcessed++
            if casesProcessed == caseData().length
               console.log "Cases: ", insertedCases
               addCustodians()

   addCustodians = =>
      console.log "\n"
      console.log "*** ADDING CUSTODIANS ***".green
      casesProcessedWithCustodian = 0
      for key, value of insertedCases
         for custodian in custodianData()
            Request {
               url: "#{API_URL}/cases/#{value}/custodians?sessionToken=#{@sessionToken}"
               method: "POST"
               body: {
                  first_name: custodian.first_name
                  last_name: custodian.last_name
                  email: custodian.email
               }
               json: true
            }, (err, res, body) ->
               console.log "err: ", err if err
               throw err if err
               insertedCustodians[body.id] = body.case_id
               casesProcessedWithCustodian++
               if casesProcessedWithCustodian == caseData().length
                  console.log "Custodians: ", insertedCustodians
                  createActivity()

   createActivity = =>
      console.log "\n"
      console.log "*** ADDING ACTIVITY ***".green
      activityProcessed = 0
      for activity in activityData()
         for caseNumber in activity.lookup_case_number
            caseId = insertedCases[caseNumber]
            ownerId = insertedUsers[activity.lookup_email]
            Request {
               url: "#{API_URL}/cases/#{caseId}/activity?sessionToken=#{@sessionToken}"
               method: "POST"
               body: {
                  owner_id: ownerId
                  message: activity.message
               }
               json: true
            }, (err, res, body) ->
               console.log "err: ", err if err
               throw err if err
               activityProcessed++ 
               if activityProcessed == activityData().length
                  console.log "Finished activity data"
                  createNewSet("Fisrt Set")

   createNewSet = (name) =>
      console.log  "\n"
      console.log "*** CREATING SET ***".green
      uploadSetCount++
      caseId = insertedCases['456456']
      Request {
         url: "#{API_URL}/cases/#{caseId}/sets?sessionToken=#{@sessionToken}"
         method: "POST"
         body: {name: name}
         json: true
      }, (err, res, body) ->
         console.log("new set error: ", err) if err
         eligableCustodians = []
         for key, value of insertedCustodians
            if value == caseId
               eligableCustodians.push(key)
         console.log "Eligable Custodians: ", eligableCustodians
         custodianId = eligableCustodians[uploadSetCount]
         createBatchSession(caseId, body.id, custodianId)

   createBatchSession = (caseId, setId, custodianId) =>
      console.log  "\n"
      console.log "*** CREATING BATCH SESSION ***".green
      Request {
         url: "#{API_URL}/cases/#{caseId}/sets/#{setId}/upload/sessions?sessionToken=#{@sessionToken}"
         method: "POST"
         body: {custodian_id: custodianId}
         json: true
      }, (err, res, body) ->
         console.log("batch session error: ", err) if err
         console.log "Created batch session for custodian id: ", custodianId
         uploadDocuments(caseId, body.batch_session)

   uploadDocuments = (caseId, batchSessionToken) =>
      console.log  "\n"
      console.log '*** UPLOADING DOCUMENTS ***'.green
      path = __dirname + "/../../example"
      recursive path, (err, files) ->
         # This makes the files have differnet custodians
         thirdFilesCount = files.length / 3
         switch uploadSetCount
            when 1
               filesToProcess = files.slice(0, thirdFilesCount)
            when 2
               filesToProcess = files.slice(thirdFilesCount, (thirdFilesCount * 2))
            when 3
               filesToProcess = files.slice(((thirdFilesCount * 2) - 2), files.length)
            when 4
               filesToProcess = files.slice(6, (files.length - 4))
            when 5
               filesToProcess = files.slice(thirdFilesCount, (thirdFilesCount * 2))
            else
               filesToProcess = files
         console.log("Found files: %s", filesToProcess.length)
         console.log("Beginning upload..")
         start = Date.now()
         Async.eachLimit filesToProcess, 10, uploadFile.bind(@, caseId, batchSessionToken), (err) =>
            console.log("error: ", err) if err
            console.log("Success! Took: %s s", ((Date.now() - start) / 1000).toFixed(2))
            if uploadSetCount == 5
               processContributors()
            else
               createNewSet("Set Number #{uploadSetCount}")

   uploadFile = (caseId, batchSessionToken, filename, callback) ->
      fs.stat filename, (err, stats) ->
         return callback(err) if err
         stream = fs.createReadStream(filename)
         ranNum = Math.floor(Math.random() * (25 - 1 + 1)) + 1;
         if ranNum <= Math.floor(tags.length / 2)
            tagsForDoc = tags.slice(0, ranNum)
         else
            tagsForDoc = tags.slice(ranNum, tags.length)
         Document.computeChecksum stream, stats.size, (err, checksum) ->
            return callback(err) if err
            data = {
               file_path: Path.resolve(filename)
               created_at: stats.birthtime.getTime()
               modified_at: stats.mtime.getTime()
               checksum: checksum
               tags: JSON.stringify(tagsForDoc)
            }
            Request {
               url: "#{UPLOAD_API_URL}/cases/#{caseId}/files/#{checksum}"
               method: "PUT"
               auth: {bearer: batchSessionToken}
               body: data
               json: true
            }, (err, res, body) ->
               return callback(err) if err
               return callback() if res.statusCode == 200
               return callback(Error(body?.message)) if res.statusCode != 404

               # We now know that this is a new file, let's upload the actual file.
               data.file = fs.createReadStream(filename)
               Request({
                  url: "#{UPLOAD_API_URL}/cases/#{caseId}/files"
                  method: "POST"
                  auth: {bearer: batchSessionToken}
                  formData: data
               }, callback)

   processContributors = =>
      console.log "\n"
      console.log "*** ADDING CONTRIBUTORS ***".green
      # TODO: this is a tmp fix. Auth needs to be normalized
      if @auth.user?
         authedUserId = @auth.user.id
      else 
         authedUserId = @auth.id
      for key, caseId of insertedCases
         switch key
            when "123123" then addContributors(caseId, [
               insertedUsers["paulm@houseatreides.com"], 
               insertedUsers["duncan.i@swordmaster.me"],
               authedUserId])
            when "234234" then addContributors(caseId, [
               insertedUsers["jsnow@nightswatch.org"], 
               insertedUsers["dog@kingsgaurd.org"],
               authedUserId])
            when "345345" then addContributors(caseId, [insertedUsers["ewiggin@xenocide.me"]])
            when "456456" then addContributors(caseId, [
               insertedUsers["ewiggin@xenocide.me"], 
               insertedUsers["jsnow@nightswatch.org"],
               insertedUsers["paulm@houseatreides.com"],
               authedUserId])
            else
               allContributors = [authedUserId]
               for email, userId in insertedUsers
                  allContributors.push(userId)
               addContributors(caseId, allContributors)

   addContributors = (caseId, contributorIds) =>
      for contributor in contributorIds
         Request {
            url: "#{API_URL}/cases/#{caseId}/contributor?sessionToken=#{@sessionToken}"
            method: "POST"
            body: {
               contributor_id: contributor
            }
            json: true
         }, (err, res, body) ->
            console.log "err: ", err if err
            throw err if err
