insist = require("insist-types")
Prompt = require("prompt")
Request = require("request")
Utils = require("../utils")
recursive = require("recursive-readdir")
Document = require("@hunterbrowning/ruleone-models").Document
Cookie = require("cookie")
fs = require("fs")
Async = require("async")
Path = require("path")

API_URL = process.env.RO_API_URL
UPLOAD_API_URL = process.env.RO_UPLOAD_API_URL

module.exports = (path) ->
   Utils.login (err, sessionToken) ->
      throw err if err
      return unless sessionToken
      schema = {properties: {case_id: {required: true}}}
      Prompt.start();
      Prompt.get schema, (err, result) ->
         process.exit() if err
         caseId = result.case_id
         getSetId sessionToken, caseId, (err, setId) ->
            throw err if err

            # Gets the custodian id.
            schema = {properties: {"custodian_id (optional)": true}}
            Prompt.start();
            Prompt.get schema, (err, result) ->
               return process.exit() if err
               custodianId = result["custodian_id (optional)"] or null
               Request {
                  url: "#{API_URL}/cases/#{caseId}/sets/#{setId}/upload/sessions?sessionToken=#{sessionToken}"
                  method: "POST"
                  body: {custodian_id: custodianId}
                  json: true
               }, (err, res, body) ->
                  throw err if err
                  return Utils.printResponse(res, body) unless res.statusCode == 200
                  uploadFiles caseId, body.batch_session, path, (err) ->
                     console.error(err.stack) if err

getSetId = (sessionToken, caseId, callback) ->
   insist.args(arguments, String, String, Function)
   schema = {properties: {"set_id (blank for new)": true}}
   Prompt.start();
   Prompt.get schema, (err, result) ->
      process.exit() if err
      return callback(null, setId) if setId = result["set_id (blank for new)"]
      createNewSet(sessionToken, caseId, callback)

createNewSet = (sessionToken, caseId, callback) ->
   insist.args(arguments, String, String, Function)
   schema = {
      properties: {
         "set name": {
            pattern: /^[a-zA-Z\s\-]+$/,
            message: 'Name must be only letters, spaces, or dashes',
            required: true
         }
      }
   }
   Prompt.start();
   Prompt.get schema, (err, result) ->
      process.exit() if err
      Request {
         url: "#{API_URL}/cases/#{caseId}/sets?sessionToken=#{sessionToken}"
         method: "POST"
         body: {name: result["set name"]}
         json: true
      }, (err, res, body) ->
         if res.statusCode != 200
            Utils.printResponse(res, body)
            process.exit()
         console.log("Created set: %s", body.id)
         callback(null, body.id)

uploadFiles = (caseId, batchSession, path, callback) ->
   insist.args(arguments, String, String, String, Function)
   recursive path, (err, files) ->
      return callback(err) if err
      console.log("Found files: %s", files.length)
      console.log("Beginning upload..")
      start = Date.now()
      Async.eachLimit files, 10, uploadFile.bind(@, caseId, batchSession), (err) ->
         return callback(err) if err
         console.log()
         console.log("Success! Took: %s s", ((Date.now() - start) / 1000).toFixed(2))
         console.log()
         callback()

uploadFile = (caseId, batchSession, filename, callback) ->
   fs.stat filename, (err, stats) ->
      return callback(err) if err
      stream = fs.createReadStream(filename)
      Document.computeChecksum stream, stats.size, (err, checksum) ->
         return callback(err) if err
         data = {
            file_path: Path.resolve(filename)
            created_at: stats.birthtime.getTime()
            modified_at: stats.mtime.getTime()
            checksum: checksum
         }
         Request {
            url: "#{UPLOAD_API_URL}/cases/#{caseId}/files/#{checksum}"
            method: "PUT"
            auth: {bearer: batchSession}
            body: data
            json: true
         }, (err, res, body) ->
            return callback(err) if err
            return callback() if res.statusCode == 200
            return callback(Error(body?.message)) if res.statusCode != 404

            # We now know that this is a new file, let's upload the actual file.
            data.file = fs.createReadStream(filename)
            Request({
               url: "#{UPLOAD_API_URL}/cases/#{caseId}/files"
               method: "POST"
               auth: {bearer: batchSession}
               formData: data
            }, callback)
